unit MSG;

interface

 uses Types, Config, Log, Messages, Init, StdFunc, Errors, SysUtils,
{$ifndef Win32}
  Use32
{$else}
  Windows
{$endif};

 const
  Origin  : string [11] = ' * Origin: ';
  TierLine: string [3]	= '---';
  SeenBy  : string [9]	= 'SEEN-BY: ';
  Path	  : string [7]	= #1'PATH: ';

 type
  pString = ^string;

  tFH = record
   case boolean of
    False: (FileHead: {$ifndef Win32}text{$else}textfile{$endif});
    True : (BinHead: file{ of byte});
  end;

  pBase = ^tBase;
  tBase = object
   TS	   : tTimeStamp;
   Head    : pBuf;
   Len	   : word;
   Error   : boolean;
   ID	   : pString;
   FileName: pString;
   constructor Init (HeadLen: word; FlNm: string);
   destructor  Done;
   procedure   Run;					     virtual;
   function    GetDT: string;
   procedure   SetRWFlag;
   procedure   ClearRWFlag;
   procedure   OpenBase (var FH: tFH);			      virtual;
   procedure   WriteHead;				      virtual;
   procedure   WriteKludge;				      virtual;
   procedure   WriteINTL;				      virtual;
   function    GenMSGID: string;
   procedure   WriteMSGID;				      virtual;
   function    GenPID: string;
   procedure   WritePID;				      virtual;
   procedure   WriteTID;				      virtual;
   procedure   WriteFMPT;				      virtual;
   procedure   WriteTOPT;				      virtual;
   procedure   WriteData;				      virtual;
   procedure   WriteOrigin;				      virtual;
   procedure   WriteSeenBy;				      virtual;
   procedure   WritePath;				      virtual;
   procedure   WriteString (Str: string);		      virtual;
   procedure   CloseBase;				      virtual;
   procedure   OpenBBase (var FH: tFH); 		      virtual;
   function    ReadBData (Size: word; var Buf): longint;      virtual;
   function    WriteBData (Size: word; var Buf): longint;     virtual;
   function    SeekBPoint (Origin, Offset: longint): longint; virtual;
   procedure   CloseBBase;				      virtual;
   procedure   GetFileHead (var FH: tFH);
   procedure   SetFileHead (FH: tFH);
  private
   FileHead: tFH;
   RWState : boolean;
   function    ConvDT (D: word): string;
   procedure   GenerateID;
  end;

 var
  TmpFH: tFH;

 procedure AddString (Data: string);
 procedure WriteReport;

implementation

 uses PKT, MSG_, MSGB;

 const
  Prg_	: string [9]  = #1'Program:';
//  Copyr_: string [12] = #1'Copyright: ';
  kINTL : string [5]  = #1'INTL';
  kMSGID: string [7]  = #1'MSGID:';
  kPID	: string [16] = #1'PID: MQR'+
{$ifdef Win32}
  '32'
{$endif}
{$ifdef OS2}
  '/2'
{$endif} +' 2.0'#13;
  kFMPT : string [6]  = #1'FMPT ';
  kTOPT : string [6]  = #1'TOPT ';

 var
  R	 : pStr_;
  Base	 : pBase;
  PKTBase: pPKT;
  NMBase : pNetMail;
  MSGBase: pMSG;
  SR	 : tSearchRec;
  Post	 : boolean;

 procedure AddString (Data: string);
 begin
{$ifndef Win32}
  if not CheckMemory (MemAvail, SizeOf (tStr)) then begin
   ToLog (0, NoMem);
   ToLog (0, InModule_ + TemplateGen_);
   DoneAllModules;
   ErrHalt (esNoMemory, '');
  end;
{$endif}
  if R = nil then begin
   R:= New (pStr_);
   R^.Prev:= nil;
  end else begin
   R^.Next:= New (pStr_);
   R^.Next^.Prev:= R;
   R:= R^.Next;
  end;
  R^.Next:= nil;
  R^.Str:= Data + #13;
 end;

 procedure WriteReport;
  procedure UseBase;
  begin
   Post:= False;
   Base:= New (pBase);
   Base^.Init (0, CFG^.Area.Path);
   Base^.Run;
   Dispose (Base, Done);
  end;
  procedure UsePKT;
  begin
   PKTBase:= New (pPKT);
   PKTBase^.Init;
   PKTBase^.Run;
   Dispose (PKTBase, Done);
  end;
  procedure UseNetMail;
  begin
   NMBase:= New (pNetMail);
   NMBase^.Init;
   NMBase^.Run;
   Dispose (NMBase, Done);
  end;
  procedure UseMSGB;
  begin
   MSGBase:= New (pMSG);
   MSGBase^.Init;
   MSGBase^.Run;
   Dispose (MSGBase, Done);
  end;
 begin
  ToLog (0, WriteReport_);
  Post:= True;
  case CFG^.Area.MSGType of
   1: UsePkt;
   4: UseNetMail;
   5: UseMSGB;
   7: UseBase;
  end;
  ToLog (0, Complete_);
 end;

 constructor tBase.Init;
 var i: word;
 begin
  FileName:= New (pString);
  FileName^:= FlNm;
  Error:= False;
  Head:= nil;
  if R = nil then Error:= True
	     else while R^.Prev <> nil do R:= R^.Prev;
  Len:= HeadLen;
  SetRWFlag;
  if Len = 0 then Exit;
{$ifndef Win32}
  if not CheckMemory (MemAvail, HeadLen) then begin
   Error:= True;
   Exit;
  end;
{$endif}
  GetMem (Head, HeadLen);
  FillChar (Head^ [0], Len, 0);
 end;

 destructor tBase.Done;
 begin
  if Head <> nil then FreeMem (Head, Len);
  if R = nil then Exit;
  while R^.Prev <> nil do R:= R^.Prev;
  while R^.Next <> nil do begin
   R:= R^.Next;
   Dispose (R^.Prev);
  end;
  Dispose (R);
  Dispose (FileName);
 end;

 procedure tBase.Run;
 begin
  if Error then Exit;
  OpenBase (TmpFH);
  if not Error then WriteHead;
  if not Error then WriteKludge;
  if not Error then WriteData;
  if not Error then WriteOrigin;
  if not Error then CloseBase;
 end;

 function tBase.GetDT: string;
 var s: string;
     t, z: longint;
 begin
  ShortDateFormat:= 'dd mmm yy';
  LongTimeFormat:= 'hh:mm:ss';
  s:= DateToStr (Date) + '  ' + TimeToStr (Time);
{1$ifdef Win32}
//  for z:= 1 to 12 do if Copy (s, 3, 3) = RMonth [z] then s:= Copy (s, 1, 3) + Month [z] + Copy (s, 7, 3);
{1$endif}
  GetDT:= s;
  t:= DateTimeToFileDate (Now);
  z:= t shr 16;
  TS.Time:= t and $FFFF;
  TS.Date:= z;
 end;

 procedure tBase.OpenBase;
 var Res: word;
 begin
  Assign (FileHead.FileHead, FileName^);
  Res:= FindFirst (FileName^, faArchive, SR);
  if Res <> 0 then Rewrite (FileHead.FileHead)
  else
   if RWState then Rewrite (FileHead.FileHead)
	      else Append (FileHead.FileHead);
  FH:= FileHead;
  if IOResult <> 0 then Error:= True;
  if R <> nil then while R^.Prev <> nil do R:= R^.Prev;
 end;

 procedure tBase.WriteHead;
 var i: word;
 begin
  if Len = 0 then Exit;
  for i:= 0 to Len - 1 do Write (FileHead.FileHead, char (Head^ [i]));
 end;

 procedure tBase.WriteKludge;
 begin
  if not Post then Exit;
  Write (FileHead.FileHead, Prg_ + PrgName + PrgType + PrgVer + PrgBits + #13);
//  Write (FileHead.FileHead, Copyr_ + PrgAut + #13);
 end;

 procedure tBase.WriteINTL;
  function Addr (A: t4D): string;
  var s, z: string;
      i   : byte;
  begin
  s:= '';
  for i:= 0 to 2 do begin
   case i of
    0: s:= s + ' ';
    1: s:= s + ':';
    2: s:= s + '/';
   end;
   Str (A [i], z);
   s:= s + z;
  end;
  Addr:= s;
 end;
 begin
  Write (FileHead.FileHead, kINTL + Addr (CFG^.ToAddress) + Addr (CFG^.FromAddress) + #13);
 end;

 function tBase.GenMSGID;
 var s, z: string;
     i	 : byte;
 begin
  GenerateID;
  s:= ' ';
  for i:= 0 to 3 do begin
   Str (CFG^.FromAddress [i], z);
   s:= s + z;
   case i of
    0: s:= s + ':';
    1: s:= s + '/';
    2: s:= s + '.';
    3: s:= s + ' ';
   end;
  end;
  GenMSGID:= kMSGID + s + ID^ + #13;
  Dispose (ID);
 end;

 procedure tBase.WriteMSGID;
 begin
  Write (FileHead.FileHead, GenMSGID);
 end;

 function tBase.GenPID;
 begin
  GenPID:= kPID;
 end;

 procedure tBase.WritePID;
 begin
  Write (FileHead.FileHead, kPID);
 end;

 procedure tBase.WriteTID;
 begin
 end;

 procedure tBase.WriteFMPT;
 begin
  if CFG^.FromAddress [3] <> 0 then Write (FileHead.FileHead, kFMPT, CFG^.FromAddress [3], #13);
 end;

 procedure tBase.WriteTOPT;
 begin
  if CFG^.ToAddress [3] <> 0 then Write (FileHead.FileHead, kTOPT, CFG^.ToAddress [3], #13);
 end;

 procedure tBase.WriteData;
 begin
  while R^.Prev <> nil do R:= R^.Prev;
  while R^.Next <> nil do begin
   Write (FileHead.FileHead, R^.Str);
   if not Post then Write (FileHead.FileHead, #10);
   R:= R^.Next;
  end;
  Write (FileHead.FileHead, R^.Str);
  if not Post then Write (FileHead.FileHead, #10);
 end;

 procedure tBase.WriteOrigin;
 var z, s: string;
     i	 : byte;
 begin
  if not Post then Exit;
  Write (FileHead.FileHead, #13 + TierLine + #13);
  s:= '(';
  s:= s + ShowAddr (CFG^.FromAddress);
  s:= s + ')';
  if Length (Origin + s + CFG^.Origin) > 77 then z:= Copy (CFG^.Origin, 1, 78 - Length (Origin + s))
					    else z:= CFG^.Origin;
  Write (FileHead.FileHead, Origin + z + ' ' + s + #13);
 end;

 procedure tBase.WriteSeenBy;
 begin
  Write (FileHead.FileHead, SeenBy, CFG^.FromAddress [1], '/', CFG^.FromAddress [2], #13);
 end;

 procedure tBase.WritePath;
 begin
  Write (FileHead.FileHead, Path, CFG^.FromAddress [1], '/', CFG^.FromAddress [2], #13);
 end;

 procedure tBase.CloseBase;
 begin
  Close (FileHead.FileHead);
 end;

 procedure tBase.WriteString;
 begin
  Write (FileHead.FIleHead, Str);
 end;

 procedure tBase.SetRWFlag;
 begin
  RWState:= True;
 end;

 procedure tBase.ClearRWFlag;
 begin
  RWState:= False;
 end;

 procedure tBase.OpenBBase;
 var Res: word;
 begin
  Assign (FileHead.BinHead, FileName^);
  Res:= FindFirst (FileName^, faArchive, SR);
  if Res <> 0 then Rewrite (FileHead.BinHead)
  else
   if RWState then Rewrite (FileHead.BinHead)
	      else Reset (FileHead.BinHead);
  FH:= FileHead;
  if IOResult <> 0 then Error:= True;
 end;

 function tBase.ReadBData;
 var Res: integer;
 begin
  BlockRead (FileHead.BinHead, Buf, Size, Res);
  ReadBData:= Res;
 end;

 function tBase.WriteBData;
 var Res: integer;
 begin
  BlockWrite (FileHead.BinHead, Buf, Size, Res);
  WriteBData:= Res;
 end;

 function tBase.SeekBPoint;
 var P: longint;
 begin
  P:= FilePos (FileHead.BinHead);
  case Origin of
   1: Seek (FileHead.BinHead, P + Offset);
   0: Seek (FileHead.BinHead, Offset);
   2: Seek (FileHead.BinHead, FileSize (FileHead.BinHead) - P);
  end;
  SeekBPoint:= FilePos (FileHead.BinHead);
 end;

 procedure tBase.CloseBBase;
 begin
  Close (FileHead.BinHead);
 end;

 procedure tBase.GetFileHead;
 begin
  FH:= FileHead;
 end;

 procedure tBase.SetFileHead;
 begin
  FileHead:= FH;
 end;

 function tBase.ConvDT (D:word): string;
 var z: string;
 begin
  Str (D, z);
  case Length (z) of
  1: z:= '0' + z;
  4: z:= Copy (z, 3, 2);
  end;
  ConvDT:= z;
 end;

 procedure tBase.GenerateID;
 var z	 : byte;
     i	 : byte;
     n	 : longint;
 begin
  n:= DateTimeToFileDate (Now);
  ID:= New (pString);
  ID^:= '';
  z:= $0F;
  for i:= 3 downto 0 do begin
   z:= (n shr (8 * i)) and $FF;
   case (z and $F0) shr 4 of
    0..9  : ID^:= ID^ + char ((z and $F0) shr 4 + $30);
    10..15: ID^:= ID^ + char ((z and $F0) shr 4 + 55);
   end;
   case z and $0F of
    0..9  : ID^:= ID^ + char (z and $0F + $30);
    10..15: ID^:= ID^ + char (z and $0F + 55);
   end;
  end;
 end;

initialization
 R:=  nil;
end.

