unit Template;

interface

 uses Config, Queue, Types, StdFunc, Log, Messages, Errors, Init, FileNms, MSG, SysUtils, VPSysLow,
{$ifndef Win32}
  VPUtils, OS2Def, OS2Base
{$else}
  Windows
{$endif};

 const
{Aliases for Template file}
  tIf	       : string [2]  = 'IF';
  cIf	       {: byte} = $01;
  tEnd	       : string [3]  = 'END';
  cEnd	       {: byte} = $00;
  tQueue       : string [5]  = 'QUEUE';
  cQueue       {: byte} = $10;
  tVersion     : string [7]  = 'VERSION';
  cVersion     {: byte} = $11;
  tCurTime     : string [4]  = 'TIME';
  cCurTime     {: byte} = $12;
  tCurDate     : string [4]  = 'DATE';
  cCurDate     {: byte} = $13;
  tDayOfWeek   : string [9]  = 'DAYOFWEEK';
  cDayOfWeek   {: byte} = $14;
  tMainAka     : string [7]  = 'MAINAKA';
  cMainAka     {: byte} = $15;
  tEqu	       : string [1]  = '=';
  cEqu	       {: byte} = $20;
  tSmall       : string [1]  = '<';
  cSmall       {: byte} = $21;
  tBig	       : string [1]  = '>';
  cBig	       {: byte} = $22;
  tSmallBig    : string [2]  = '<>';
  cSmallBig    {: byte} = $23;
  tEquSmall    : string [2]  = '<=';
  cEquSmall    {: byte} = $24;
  tEquBig      : string [2]  = '>=';
  cEquBig      {: byte} = $25;
  tQueueBegin  : string [10] = 'QUEUEBEGIN';
  cQueueBegin  {: byte} = $02;
  tNode        : string [4]  = 'NODE';
  cNode        {: byte} = $30;
  tMSize       : string [5]  = 'MSIZE';
  cMSize       {: byte} = $31;
  tTSize       : string [5]  = 'TSIZE';
  cTSize       {: byte} = $32;
  tPercent     : string [7]  = 'PERCENT';
  cPercent     {: byte} = $33;
  tFlags       : string [5]  = 'FLAGS';
  cFlags       {: byte} = $34;
  tFDays       : string [5]  = 'FDAYS';
  cFDays       {: byte} = $35;
  tFromDay     : string [7]  = 'FROMDAY';
  cFromDay     {: byte} = $36;
  tCDays       : string [5]  = 'CDAYS';
  cCDays       {: byte} = $37;
  tCFiles      : string [6]  = 'CFILES';
  cCFiles      {: byte} = $38;
  tTFiles      : string [6]  = 'TFILES';
  cTFiles      {: byte} = $39;
  tCDayTime    : string [8]  = 'CDAYTIME';
  cCDayTime    {: byte} = $3A;
  tPFlags      : string [6]  = 'PFLAGS';
  cPFlags      {: byte} = $3B;
  tDrvInfoBegin: string [12] = 'DRVINFOBEGIN';
  cDrvInfoBegin{: byte} = $03;
  tDLetter     : string [7]  = 'DLETTER';
  cDLetter     {: byte} = $40;
  tDSize       : string [5]  = 'DSIZE';
  cDSize       {: byte} = $41;
  tDFree       : string [5]  = 'DFREE';
  cDFree       {: byte} = $42;
  tDLabel      : string [6]  = 'DLABEL';
  cDLabel      {: byte} = $43;
  tDFS	       : string [3]  = 'DFS';
  cDFS	       {: byte} = $44;
  tDPercent    : string [8]  = 'DPERCENT';
  cDPercent    {: byte} = $45;

 var
  Tpl: pStr_;
  Rep: pStr_;

 procedure CreateReport;
 procedure CloseReport;

implementation

{$ifdef Win32}
 type
  tGDFSEA = function (Root: pointer; var FBC, TB, FB: longint): longbool;

 var
  GetDiskFreeSpaceExA: tGDFSEA;
  GDFSEA	     : boolean;
{$endif}

 const
  TotalMailSize    : comp = 0;
  TotalNetMailSize : comp = 0;
  TotalArcMailSize : comp = 0;
  TotalFilesSize   : comp = 0;
  TotalMailFiles   : word = 0;
  TotalNetMailFiles: word = 0;
  TotalArcMailFiles: word = 0;
  TotalFilesFiles  : word = 0;
 var
  Data, e	   : string;
  e1		   : {$ifdef Win32}dword{$else}ULong{$endif};
  e2		   : {$ifdef Win32}dword{$else}ULong{$endif};
  DriveLetter	   : char;
  ChengetRec	   : boolean;

 function DriveType: string;
{$ifdef Win32}
 const Root: array [0..4] of char = 'C:\'#0;
 var FSName: array [0..255] of char;
     MaxLength: longint;
     FSFlags  : longint;
 begin
  Root [0]:= DriveLetter;
  if GetVolumeInformation (Root, nil, 0, nil, MaxLength, FSFlags, FSName, sizeof (FSName)) then DriveType:= StrPas (FSName)
											   else DriveType:= 'Unknown';
 end;
{$else}
 var BufLen    : longint;
     FSQb      : pFSQBuffer2;
     DrvName   : string [3];
     Ordinal   : word;
     name      : pChar;
     rc        : longint;
 begin
  BufLen:= 100;
  GetMem (FSQb, BufLen);
  DrvName:= DriveLetter + ':'#0;
  Ordinal:= 0;
  rc:= DosQueryFSAttach (@DrvName[1], Ordinal, fsail_QueryName, FSqb, BufLen);
  if rc = 0 then with FsqB^ do begin
   Name:= szName + cbName + 1;
   DriveType:= StrPas (Name);
  end else DriveType:= 'Unknown';
  FreeMem (FSQb, 100);
 end;
{$endif}

 procedure CalculateTotalMailSize;
 begin
  if Q = nil then Exit;
  while Q^.Next <> nil do begin
   TotalArcMailSize:= TotalArcMailSize + Q^.AM;
   TotalNetMailSize:= TotalNetMailSize + Q^.NM;
   TotalFilesSize:= TotalFilesSize + Q^.FM;
   TotalNetMailFiles:= TotalNetMailFiles + Q^.NF;
   TotalArcMailFiles:= TotalArcMailFiles + Q^.AF;
   TotalFilesFiles:= TotalFilesFiles + Q^.FF;
   Data:= For_ + ShowAddr (Q^.Addr) + NetMail_ + DWordToStr (Q^.NF) +
				      ArcMail_ + DWordToStr (Q^.AF) +
				      FMail_ + DWordToStr (Q^.FF) + ' => ' +
				      GetDateTime (Q^.Date.Y, Q^.Date.M, Q^.Date.D, Q^.Time.H, Q^.Time.M, Q^.Time.S);
   ToLog (1, Data);
   Q:= Q^.Next;
  end;
  TotalNetMailSize:= TotalNetMailSize + Q^.NM;
  TotalArcMailSize:= TotalArcMailSize + Q^.AM;
  TotalFilesSize:= TotalFilesSize + Q^.FM;
  TotalNetMailFiles:= TotalNetMailFiles + Q^.NF;
  TotalArcMailFiles:= TotalArcMailFiles + Q^.AF;
  TotalFilesFiles:= TotalFilesFiles + Q^.FF;
  Data:= For_ + ShowAddr (Q^.Addr) + NetMail_ + DWordToStr (Q^.NF) +
				     ArcMail_ + DWordToStr (Q^.AF) +
				     FMail_ + DWordToStr (Q^.FF);
  ToLog (1, Data);
  TotalMailSize:= TotalNetMailSize + TotalArcMailSize + TotalFilesSize;
  TotalMailFiles:= TotalNetMailFiles + TotalArcMailFiles + TotalFilesFiles;
  ToLog (0, TotalMail + SizeToStrD (TotalMailSize, '.', 0) + Bytes_ + In_ + SizeToStrD (TotalMailFiles, '.', 0) + Files_);
  while Q^.Prev <> nil do Q:= Q^.Prev;
 end;

 function Command (Data: string; Index: byte; var ComType: byte): boolean;
 begin
  Command:= False;
  ComType:= $FF;
  if Length (Data) <= Index + 1 then Exit;
  if Copy (Data, Index + 1, 3)	= tEnd		then ComType:= cEnd;
  if Copy (Data, Index + 1, 2)	= tIf		then ComType:= cIf;
  if Copy (Data, Index + 1, 3)	= tEnd		then ComType:= cEnd;
  if Copy (Data, Index + 1, 5)	= tQueue	then ComType:= cQueue;
  if Copy (Data, Index + 1, 7)	= tVersion	then ComType:= cVersion;
  if Copy (Data, Index + 1, 4)	= tCurTime	then ComType:= cCurTime;
  if Copy (Data, Index + 1, 4)	= tCurDate	then ComType:= cCurDate;
  if Copy (Data, Index + 1, 9)	= tDayOfWeek	then ComType:= cDayOfWeek;
  if Copy (Data, Index + 1, 7)	= tMainAka	then ComType:= cMainAka;
  if Copy (Data, Index + 1, 10) = tQueueBegin	then ComType:= cQueueBegin;
  if Copy (Data, Index + 1, 4)	= tNode 	then ComType:= cNode;
  if Copy (Data, Index + 1, 5)	= tMSize	then ComType:= cMSize;
  if Copy (Data, Index + 1, 5)	= tTSize	then ComType:= cTSize;
  if Copy (Data, Index + 1, 7)	= tPercent	then ComType:= cPercent;
  if Copy (Data, Index + 1, 5)	= tFlags	then ComType:= cFlags;
  if Copy (Data, Index + 1, 5)	= tFDays	then ComType:= cFDays;
  if Copy (Data, Index + 1, 5)	= tCDays	then ComType:= cCDays;
  if Copy (Data, Index + 1, 7)	= tFromDay	then ComType:= cFromDay;
  if Copy (Data, Index + 1, 8)	= tCDayTime	then ComType:= cCDayTime;
  if Copy (Data, Index + 1, 6)	= tPFlags	then ComType:= cPFlags;
  if Copy (Data, Index + 1, 12) = tDrvInfoBegin then ComType:= cDrvInfoBegin;
  if Copy (Data, Index + 1, 7)	= tDLetter	then ComType:= cDLetter;
  if Copy (Data, Index + 1, 5)	= tDSize	then ComType:= cDSize;
  if Copy (Data, Index + 1, 5)	= tDFree	then ComType:= cDFree;
  if Copy (Data, Index + 1, 6)	= tDLabel	then ComType:= cDLabel;
  if Copy (Data, Index + 1, 6)	= tCFiles	then ComType:= cCFiles;
  if Copy (Data, Index + 1, 6)	= tTFiles	then ComType:= cTFiles;
  if Copy (Data, Index + 1, 3)	= tDFS		then ComType:= cDFS;
  if Copy (Data, Index + 1, 8)	= tDPercent	then ComType:= cDPercent;
  if ComType <> $FF then Command:= True;
 end;

 procedure AnalyzeString (Data: string; A: boolean); forward;

 function Execute_if (Data: string; Index: byte; A: boolean): boolean;
 var D1, D2, D3: string;
     j, i: byte;
  procedure Execute (State: boolean);
  begin
   while (D1 <> '@' + tEnd) and (Tpl^.Next <> nil) do begin
    if not ChengetRec then Tpl:= Tpl^.Next;
    ChengetRec:= False;
    AnalyzeString (Tpl^.Str, State);
    D1:= UpStr (Tpl^.Str);
   end;
  end;
 begin
  Execute_if:= False;
  if Data [Index + 3] <> '(' then Exit;
  i:= 0;
  j:= Index + 4;
  while (Data [j + i] <> ',') and (Length (Data) >= j + i) do Inc (i);
  if (Data [j + i] = ',') and (Length (Data) > j + i) then D1:= Copy (Data, j, i)
  else Exit;
  j:= j + i + 1;
  i:= 0;
  while (Data [j + i] <> ',') and (Length (Data) >= j + i) do Inc (i);
  if (Data [j + i] = ',') and (Length (Data) > j + i) then D2:= Copy (Data, j, i)
  else Exit;
  j:= j + i + 1;
  i:= 0;
  while (Data [j + i] <> ')') and (Length (Data) >= j + i) do Inc (i);
  if (Data [j + i] = ')') and (Length (Data) >= j + i) then D3:= Copy (Data, j, i)
  else Exit;
  if D1 [1] <> '@' then Exit;
  D1:= UpStr (D1);
  if Copy (D1, 2, Length (D1) - 1) <> tQueue then Exit;
  if (Length (D2) > 2) and
     (D2 <> tEqu) and
     (D2 <> tSmall) and
     (D2 <> tBig) and
     (D2 <> tSmallBig) and
     (D2 <> tEquSmall) and
     (D2 <> tEquBig) then Exit;
  Val (D3, e1, e2);
  Str (e1, e);
  if e <> D3 then Exit;
  Execute_if:= True;
  if not A then Execute (False)
  else if (D2 = tEqu) and (QCR = e1) then Execute (True)
  else if (D2 = tSmall) and (QCR < e1) then Execute (True)
  else if (D2 = tBig) and (QCR > e1) then Execute (True)
  else if (D2 = tSmallBig) and (QCR <> e1) then Execute (True)
  else if (D2 = tEquSmall) and (QCR <= e1) then Execute (True)
  else if (D2 = tEquBig) and (QCR >= e1) then Execute (True)
  else Execute (False);
  ChengetRec:= True;
 end;

 function Execute_queuebegin (A: boolean): boolean;
 var MTpl: pStr_;
  procedure Execute;
  begin
   Data:= Tpl^.Str;
   while (Data <> '@' + tEnd) and (Tpl^.Next <> nil) do begin
    if not ChengetRec then Tpl:= Tpl^.Next;
    ChengetRec:= False;
    AnalyzeString (Tpl^.Str, A);
    Data:= UpStr (Tpl^.Str);
   end;
   if Tpl^.Next <> nil then Tpl:= Tpl^.Next;
  end;
 begin
  Execute_queuebegin:= True;
  MTpl:= Tpl;
  if Q = nil then begin
   Execute;
   Exit;
  end;
  while Q^.Next <> nil do begin
   Tpl:= MTpl;
   Execute;
   Q:= Q^.Next;
  end;
  Tpl:= MTpl;
  Execute;
  ChengetRec:= True;
 end;

 function Execute_drvinfobegin (Data: string; Index: byte; A: boolean): boolean;
 var Drvs  : string;
     j	   : byte;
     MTpl  : pStr_;
     z	   : longint;
  procedure Execute (State: boolean);
  var i: byte;
  begin
   e:= Tpl^.Str;
   if not State then begin
    while (e <> '@' + tEnd) and (Tpl^.Next <> nil) do begin
     if not ChengetRec then Tpl:= Tpl^.Next;
     ChengetRec:= False;
     e:= UpStr (Tpl^.Str);
    end;
    if Tpl^.Next <> nil then Tpl:= Tpl^.Next;
    Exit;
   end;
   MTpl:= Tpl;
   for i:= 1 to Length (Drvs) do  begin
    DriveLetter:= Drvs [i];
    Tpl:= MTpl;
    e:= Tpl^.Str;
    while (e <> '@' + tEnd) and (Tpl^.Next <> nil) do begin
     if not ChengetRec then Tpl:= Tpl^.Next;
     ChengetRec:= False;
     if DiskFree (byte (DriveLetter) - $40) <> -1 then AnalyzeString (Tpl^.Str, A);
     e:= UpStr (Tpl^.Str);
    end;
    if Tpl^.Next <> nil then Tpl:= Tpl^.Next;
   end;
  end;
  procedure AddDrive (L: char);
  var i: byte;
  begin
   if (z shr (byte (l) - $41)) and 1 = 1 then begin
    i:= 1;
    while (i <= Length (e)) and (byte (L) > byte (e [i])) do Inc (i);
    if (Length (e) > 1) and (e [i - 1] = L) then Exit;
    if i <= Length (e) then e:= Copy (e, 1, i - 1) + L + Copy (e, i, Length (e) - i + 1)
    else if i > Length (e) then e:= e + L;
   end;
  end;
  procedure AllDrives;
  var i: byte;
  begin
   for i:= $43 to $5A do AddDrive (char (i));
  end;
 begin
  z:= SysGetValidDrives;
  Execute_drvinfobegin:= False;
  Drvs:= '';
  j:= Length (tDrvInfoBegin) + 2;
  if Data [j] <> '(' then Exit;
  Inc (j);
  e:= UpStr (Data);
  while Length (Data) > j do begin
   if (Data [j + 1] <> ',') and
      (Data [j + 1] <> ')') then Exit;
   if ((byte (e [j]) >= 65) and
       (byte (e [j]) <= 90)) or
      (e [j] = '*') then begin
    Drvs:= Drvs + e [j];
    j:= j + 2;
   end else Exit;
  end;
  j:= 1;
  e:= '';
  while j <= Length (Drvs) do begin
   case Drvs [j] of
    '*'     : AllDrives;
    'A'..'Z': AddDrive (Drvs [j]);
   end;
   Inc (j);
  end;
  Drvs:= e;
  if Drvs <> '' then Execute (True)
		else Execute (False);
  Execute_drvinfobegin:= True;
  ChengetRec:= True;
 end;

 function Execute_queue (Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Add:= 5;
  Str (QCR: 3, Data);
  Execute_queue:= True;
 end;

 function Execute_version (Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Add:= 7;
  Data:= Copy (PrgVer, 4, 4);
  Execute_version:= True;
 end;

 function Execute_time (Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Add:= 4;
  LongTimeFormat:= 'HH:MM:SS';
  Data:= TimeToStr (Time);
  Execute_time:= True;
 end;

 function Execute_date (Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Add:= 4;
  ShortDateFormat:= 'DD MMM YYYY';
  Data:= DateToStr (Date);
  Execute_date:= True;
 end;

 function Execute_dayofweek (Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Add:= 9;
  Data:= LongDayNames [DayOfWeek (Date)];
  Execute_dayofweek:= True;
 end;

 function Execute_mainaka (Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Add:= 7;
  Data:= ShowAddr (CFG^.MainAddress);
  Execute_mainaka:= True;
 end;

 function Execute_node (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 var D1, D2: string;
     i: byte;
 begin
  Data:= '';
  Add:= 0;
  Execute_node:= False;
  if D [Index + 5] <> '(' then Exit;
  if D [Index + 7] <> ',' then Exit;
  D1:= UpStr (D [Index + 6]);
  if (D1 <> 'R') and (D1 <> 'L') then Exit;
  i:= 0;
  while (D [Index + 8 + i] <> ')') and (Index + 8 + i <= Length (D)) do Inc (i);
  if D [Index + 8 + i] <> ')' then Exit;
  D2:= Copy (D, Index + 8, i);
  Add:= 8 + i;
  Val (D2, e1, e2);
  Str (e1, e);
  if D2 <> e then Exit;
  D2:= ShowAddr (Q^.Addr);
  if Length (D2) > e1 then D2:= Copy (D2, 1, e1)
  else begin
   e2:= e1 - Length (D2);
   for i:= 1 to e2 do if D1 = 'R' then D2:= ' ' + D2
				  else D2:= D2 + ' ';
  end;
  Data:= D2;
  Execute_node:= True;
 end;

 function Execute_size (D: string; Index: byte; NMS, AMS, FMS: comp;
			var Add: byte; var Data: string): boolean;
 var Dat: array [1..5] of string;
     i, j: byte;
     MS: comp;
 begin
  Data:= '';
  Add:= 0;
  Execute_size:= False;
  if D [Index + 6] <> '(' then Exit;
  if D [Index + 8] <> ',' then Exit;
  Dat [1]:= UpStr (D [Index + 7]);
  if (Dat [1] <> 'L') and (Dat [1] <> 'R') then Exit;
  i:= Index + 9;
  j:= 0;
  while (D [i + j] <> ',') and (i + j <= Length (D)) do Inc (j);
  if (D [i + j] <> ',') or (j = 0) or (j > 3) then Exit;
  Dat [2]:= UpStr (Copy (D, i, j));
  i:= i + j + 1;
  for j:= 1 to Length (Dat [2]) do
   if (Dat [2,j] <> 'N') and
      (Dat [2,j] <> 'A') and
      (Dat [2,j] <> 'F') then Exit;
  if (D [i] <> ',') and (D [i + 1] <> ',') then Exit;
  if D [i + 1] = ',' then Dat [3]:= D [i]
		     else Dat [3]:= '';
  i:= i + 1 + Length (Dat [3]);
  if D [i + 1] <> ',' then Exit;
  Dat [4]:= UpStr (D [i]);
  if (Dat [4] <> 'M') and
     (Dat [4] <> 'K') and
     (Dat [4] <> 'B') then Exit;
  i:= i + 2;
  j:= 0;
  while (D [i + j] <> ')') and (i + j <= Length (D)) do Inc (j);
  if D [i + j] <> ')' then Exit;
  Dat [5]:= Copy (D, i, j);
  Add:= i + j;
  Val (Dat [5], e1, e2);
  Str (e1, e);
  if Dat [5] <> e then Exit;
  MS:= 0;
  for i:= 1 to Length (Dat [2]) do
   case Dat [2,i] of
    'N': MS:= MS + NMS;
    'A': MS:= MS + AMS;
    'F': MS:= MS + FMS;
   end;
  case Dat [4,1] of
   'B': Data:= SizeToStrD (MS, Dat [3], 0);
   'K': Data:= SizeToStrD (MS, Dat [3], 1);
   'M': Data:= SizeToStrD (MS, Dat [3], 2);
  end;
  if Length (Data) > e1 then Data:= Copy (Data, 1, e1)
  else begin
   e2:= e1 - Length (Data);
   for i:= 1 to e2 do
    if Dat [1] = 'R' then Data:= ' ' + Data
		     else Data:= Data + ' ';
  end;
  Execute_size:= True;
  Add:= 13 + Length (Dat [2] + Dat [3] + Dat [5]);
 end;

 function Execute_msize (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Execute_msize:= Execute_size (D, Index, Q^.NM, Q^.AM, Q^.FM, Add, Data);
 end;

 function Execute_tsize (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Execute_tsize:= Execute_size (D, Index, TotalNetMailSize, TotalArcMailSize,
				TotalFilesSize, Add, Data);
 end;

 function Execute_percent (var Add: byte; var Data: string): boolean;
 var s: string;
 begin
  Execute_percent:= True;
  Str (((Q^.NM + Q^.AM + Q^.FM)/TotalMailSize)*100:3:2, s);
  case Length (s) of
   4: s:= '  ' + s;
   5: s:= ' ' + s;
  end;
  s [4]:= ',';
  Data:= s;
  Add:= 7;
 end;

 function Execute_flags (var Add: byte; var Data: string): boolean;
 begin
  Execute_flags:= True;
  Add:= 5;
  Data:= Q^.Flag;
 end;

 function Execute_pflags (var Add: byte; var Data: string): boolean;
 var s: string [6];
     i: byte;
  procedure SetFlag (F: char);
  begin
   if s = '' then s:= F
   else if not (F in ['C', 'D', 'I']) or not (s [1] in ['C', 'D', 'I']) then s:= F + s
   else if s [1] <> 'I' then if (s [1] <> 'C') or (F = 'I') then s [1]:= F;
  end;
 begin
  Execute_pflags:= True;
  s:= '';
  for i:= 1 to 7 do
   if Q^.Flag [9 - i] <> ' ' then SetFlag (Q^.Flag [9 - i]);
  for i:= 1 to 5 - Length (s) do s:= ' ' + s;
  s:= Q^.Flag [1] + s;
  Add:= 6;
  Data:= s;
 end;

 function CDay (M, Y: word): byte;
 const C: array [1..12] of byte = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
 begin
  CDay:= C [M];
  if (Trunc (Y / 4) = Y /4) and (M = 2) then CDay:= 29;
 end;

 function CDays: byte;
 var Days, i: byte;
     Y, M, D: word;
 begin
  DecodeDate (Date, Y, M, D);
  Days:= 0;
  if Q^.Date.Y < Y then begin
   for i:= 1 to M - 1 do Days:= Days + CDay (i, Y);
   Days:= Days + D + CDay (Q^.Date.M, Q^.Date.Y) - Q^.Date.D + 1;
   for i:= Q^.Date.M + 1 to 12 do Days:= Days + CDay (i, Q^.Date.Y);
  end else begin
   for i:= Q^.Date.M + 1 to M - 1 do Days:= Days + CDay (i, Y);
   if Q^.Date.M < M then Days:= Days + D + CDay (Q^.Date.M, Y) - Q^.Date.D
		    else Days:= D - Q^.Date.D;
  end;
  CDays:= Days;
 end;

 function Execute_fdays (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 var D1, D2	: string;
     i, Days	: byte;
 begin
  Data:= '';
  Add:= 0;
  Execute_fdays:= False;
  if D [Index + 6] <> '(' then Exit;
  if (D [Index + 8] <> ',') and
     (D [Index + 9] <> ',') then Exit;
  if D [Index + 8] = ',' then D1:= UpStr (D [Index + 7])
			 else D1:= UpStr (Copy (D, Index + 7, 2));
  Val (D1, e1, e2);
  Str (e1, e);
  if D1 <> e then Exit;
  if e1 > 90 then e1:= 90;
  if D [Index + 8] = ',' then i:= Index + 9
			 else i:= Index + 10;
  if D [i + 1] <> ')' then Exit;
  D2:= D [i];
  e2:= i + 1;
  Days:= CDays;
  if Days <= e1 then Str (Days, D2);
  if Length (D2) = 1 then D2:= ' ' + D2;
  Data:= D2;
  Add:= e2 - Index;
  Execute_fdays:= True;
 end;

 function Execute_fromday (var Add: byte; var Data: string): boolean;
 var s: string;
 begin
  Str (Q^.Date.D, e);
  if Length (e) <> 2 then e:= '0' + e;
  s:= e + '-';
  Str (Q^.Date.M, e);
  if Length (e) <> 2 then e:= '0' + e;
  s:= s + e + '-';
  Str (Q^.Date.Y, e);
  s:= s + e;
  Add:= 7;
  Data:= s;
  Execute_fromday:= True;
 end;

 function Execute_cdays (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 var D1, D2: string;
     i: byte;
 begin
  Data:= '';
  Add:= 0;
  Execute_cdays:= False;
  if D [Index + 6] <> '(' then Exit;
  if D [Index + 8] <> ',' then Exit;
  D1:= UpStr (D [Index + 7]);
  if (D1 <> 'R') and (D1 <> 'L') then Exit;
  i:= 0;
  while (D [Index + 9 + i] <> ')') and (Index + 9 + i <= Length (D)) do Inc (i);
  if D [Index + 9 + i] <> ')' then Exit;
  D2:= Copy (D, Index + 9, i);
  Add:= 9 + i;
  Val (D2, e1, e2);
  Str (e1, e);
  if D2 <> e then Exit;
  Str (CDays, D2);
  e2:= e1 - Length (D2);
  for i:= 1 to e2 do
   if D1 = 'R' then D2:= ' ' + D2
	       else D2:= D2 + ' ';
  Data:= D2;
  Execute_cdays:= True;
 end;

 function Execute_files (D: string; Index: byte;
			NMS, AMS, FMS: {$ifdef Win32}dword{$else}ULong{$endif};
			var Add: byte; var Data: string): boolean;
 var Dat: array [1..3] of string;
     i, j: byte;
     MS: {$ifdef Win32}dword{$else}ULong{$endif};
 begin
  Data:= '';
  Add:= 0;
  Execute_files:= False;
  if D [Index + 7] <> '(' then Exit;
  if D [Index + 9] <> ',' then Exit;
  Dat [1]:= UpStr (D [Index + 8]);
  if (Dat [1] <> 'L') and (Dat [1] <> 'R') then Exit;
  i:= Index + 10;
  j:= 0;
  while (D [i + j] <> ',') and (i + j <= Length (D)) do Inc (j);
  if (D [i + j] <> ',') or (j = 0) or (j > 3) then Exit;
  Dat [2]:= UpStr (Copy (D, i, j));
  i:= i + j + 1;
  for j:= 1 to Length (Dat [2]) do
   if (Dat [2,j] <> 'N') and
      (Dat [2,j] <> 'A') and
      (Dat [2,j] <> 'F') then Exit;
  j:= 0;
  while (D [i + j] <> ')') and (i + j <= Length (D)) do Inc (j);
  if D [i + j] <> ')' then Exit;
  Dat [3]:= Copy (D, i, j);
  Add:= i + j;
  Val (Dat [3], e1, e2);
  Str (e1, e);
  if Dat [3] <> e then Exit;
  MS:= 0;
  for i:= 1 to Length (Dat [2]) do
   case Dat [2,i] of
    'N': MS:= MS + NMS;
    'A': MS:= MS + AMS;
    'F': MS:= MS + FMS;
   end;
  Str (MS, Data);
  if Length (Data) > e1 then Data:= Copy (Data, 1, e1)
  else begin
   e2:= e1 - Length (Data);
   for i:= 1 to e2 do
    if Dat [1] = 'R' then Data:= ' ' + Data
		     else Data:= Data + ' ';
  end;
  Execute_files:= True;
  Add:= 11 + Length (Dat [2] + Dat [3]);
 end;

 function Execute_cfiles (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Execute_cfiles:= Execute_files (D, Index, Q^.NF, Q^.AF, Q^.FF, Add, Data);
 end;

 function Execute_tfiles (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Execute_tfiles:= Execute_files (D, Index, TotalNetMailFiles, TotalArcMailFiles,
				 TotalFilesFiles, Add, Data);
 end;

 procedure CTime (var CDays, CHour, CMin: byte);
 var i: byte;
     Year, Month, Day: word;
     Hour, Min, Sec, MSec: word;
     Tm: word;
 begin
  DecodeDate (Date, Year, Month, Day);
  DecodeTime (Time, Hour, Min, Sec, MSec);
  CDays:= 0; CHour:= 0; CMin:= 0;
  if Q^.Date.Y < Year then begin
   for i:= 1 to Month - 1 do CDays:= CDays + CDay (i, Year);
   CDays:= CDays + Day + CDay (Q^.Date.M, Q^.Date.Y) - Q^.Date.D;
   for i:= Q^.Date.M + 1 to 12 do CDays:= CDays + CDay (i, Q^.Date.Y);
   Tm:= 1440 - Q^.Time.H * 60 - Q^.Time.M + Hour * 60 + Min;
  end else begin
   if Q^.Date.M + 1 < Month then for i:= Q^.Date.M + 1 to Month - 1 do CDays:= CDays + Cday (i, Year);
   if Q^.Date.M < Month then CDays:= CDays + Day + CDay (Q^.Date.M, Year) - Q^.Date.D
			else CDays:= Day - Q^.Date.D;
   if (Q^.Time.H * 60 + Q^.Time.M) > (Hour * 60 + Min) then begin
    Dec (CDays);
    Tm:= 1440 - (Q^.Time.H - Hour) * 60 - Q^.Time.M + Min;
   end else Tm:= (Hour - Q^.Time.H) * 60 + Min - Q^.Time.M;
  end;
  CHour:= Tm div 60;
  CMin:= Tm - CHour * 60;
  if CMin > 59 then begin
   CMin:= CMin - 60;
   Inc (CHour);
  end;
  if CHour > 23 then CHour:= CHour - 24;
 end;

 function Execute_cdaytime (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 var D1, D2: string;
     i: byte;
     D_, H_, M_: byte;
 begin
  Data:= '';
  Add:= 0;
  Execute_cdaytime:= False;
  if D [Index + 9] <> '(' then Exit;
  if (D [Index + 11] <> ',') and
     (D [Index + 12] <> ',') and
     (D [Index + 13] <> ',') then Exit;
  if D [Index + 11] = ',' then D1:= UpStr (D [Index + 10])
  else if D [Index + 12] = ',' then D1:= UpStr (Copy (D, Index + 10, 2))
  else D1:= UpStr (Copy (D, Index + 10, 3));
  Val (D1, e1, e2);
  Str (e1, e);
  if D1 <> e then Exit;
  if D [Index + 11] = ',' then i:= Index + 12
  else if D [index + 12] = ',' then i:= Index + 13
  else i:= Index + 14;
  if D [i + 1] <> ')' then Exit;
  D2:= D [i];
  e2:= i + 1;
  CTime (D_, H_, M_);
  D1:= '';
  Str (D_: 4, e);
  Str (H_, D1);
  if Length (D1) <> 2 then D1:= '0' + D1;
  if D_ <> 0 then D1:= e + '-' + D1 else D1:= '     ' + D1;
  Str (M_, e);
  if Length (e) <> 2 then e:= '0' + e;
  D1:= D1 + ':' + e;
  if D_ >= e1 then D1:= D2 + D1 else D1:= ' ' + D1;
  Add:= e2 - Index;
  Data:= D1;
  Execute_cdaytime:= True;
 end;


 function Execute_dletter (var Add: byte; var Data: string): boolean;
 begin
  Execute_dletter:= True;
  Add:= 7;
  Data:= DriveLetter;
 end;

 function Execute_dlabel (var Add: byte; var Data: string): boolean;
 begin
  e:= SysGetVolumeLabel (DriveLetter);
  if e = '' then e:= '   ' + NoLabel_;
  if Length (e) > 11 then e:= Copy (e, 1, 11);
  for e1:= 1 to 11 do if e [e1] = #0 then begin
   e:= Copy (e, 1, e1 - 1);
   break;
  end;
  for e1:= 1 to 11 - Length (e) do e:= e + ' ';
  Data:= e;
  Add:= 6;
  Execute_dlabel:= True;
 end;

 function DCalculate (CT: byte; D: string; Index: byte; var Add: byte; var Data: string): boolean;
 var Dat: array [1..4] of string;
     i, j: byte;
     MS: comp;
{$ifdef Win32}
//     FBC: array [0..1] of longint;
//     TB : array [0..1] of longint;
//     FB : array [0..1] of longint;
       FBC : comp;
       TB  : comp;
       FB  : comp;
       pFBC: longint;
       pTB : longint;
       pFB : longint;
  const FAT32: string [5] = 'FAT32';
  function DiskSize (DL: char): boolean;
  const Root: array [0..3] of char = 'C:\'#0;
  begin
   SysBeep;
   Writeln ('! '+Fat32);
   Root [0]:= DL;
   pFBC:= longint (@FBC);
   pTB:= longint (@TB);
   pFB:= longint (@FB);
   FBC:= 0; TB:= 0; FB:= 0;
   DiskSize:= GetDiskFreeSpaceExA (@Root, pFBC, pTB, pFB) <> False;
   Writeln (SizeToStrD (FBC, '', 0));
  end;
  function SysDiskSizeLong (DL: byte): comp;
  begin
   if GDFSEA then writeln ('Yes') else writeln ('Fuck!!!!!!!');
   if (Copy (DriveType, 1, 5) = FAT32) and GDFSEA then
    if DiskSize (char ($40 + DL)) then SysDiskSizeLong:= TB //[1] * 4.0 * 1024 * 1024 * 1024 + TB [0]
				  else SysDiskSizeLong:= VPSysLow.SysDiskSizeLong (DL)
   else SysDiskSizeLong:= VPSysLow.SysDiskSizeLong (DL);
  end;
  function SysDiskFreeLong (DL: byte): comp;
  begin
   if GDFSEA then writeln ('Yes') else writeln ('Fuck!!!!!!!');
   if (Copy (DriveType, 1, 5) = FAT32) and GDFSEA then
    if DiskSize (char ($40 + DL)) then SysDiskFreeLong:= FBC //[1] * 4.0 * 1024 * 1024 * 1024 + FBC [0]
				  else SysDiskFreeLong:= VPSysLow.SysDiskFreeLong (DL)
   else SysDiskFreeLong:= VPSysLow.SysDiskFreeLong (DL);
  end;
{$endif}
 begin
  Data:= '';
  Add:= 0;
  DCalculate:= False;
  if D [Index + 6] <> '(' then Exit;
  if D [Index + 8] <> ',' then Exit;
  Dat [1]:= UpStr (D [Index + 7]);
  if (Dat [1] <> 'L') and (Dat [1] <> 'R') then Exit;
  i:= Index + 9;
  j:= 0;
  if (D [i] <> ',') and (D [i + 1] <> ',') then Exit;
  if D [i + 1] = ',' then Dat [2]:= D [i]
		     else Dat [2]:= '';
  i:= i + 2;
  if D [i + 1] <> ',' then Exit;
  Dat [3]:= UpStr (D [i]);
  if (Dat [3] <> 'M') and
     (Dat [3] <> 'K') and
     (Dat [3] <> 'B') then Exit;
  i:= i + 2;
  while (D [i + j] <> ')') and (i + j <= Length (D)) do Inc (j);
  if D [i + j] <> ')' then Exit;
  Dat [4]:= Copy (D, i, j);
  Add:= i + j;
  Val (Dat [4], e1, e2);
  Str (e1, e);
  if Dat [4] <> e then Exit;
  case CT of
   1: MS:= SysDiskSizeLong (byte (DriveLetter) - $40);
   2: MS:= SysDiskFreeLong (byte (DriveLetter) - $40);
  end;
  case Dat [3,1] of
   'B': Data:= SizeToStrD (MS, Dat [2], 0);
   'K': Data:= SizeToStrD (MS, Dat [2], 1);
   'M': Data:= SizeToStrD (MS, Dat [2], 2);
  end;
  if Length (Data) > e1 then Data:= Copy (Data, 1, e1)
  else begin
   e2:= e1 - Length (Data);
   for i:= 1 to e2 do
    if Dat [1] = 'R' then Data:= ' ' + Data
		     else Data:= Data + ' ';
  end;
  DCalculate:= True;
  Add:= 12 + Length (Dat [2] + Dat [4]);
 end;

 function Execute_dsize (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Execute_dsize:= DCalculate (1, D, Index, Add, Data);
 end;

 function Execute_dfree (D: string; Index: byte; var Add: byte; var Data: string): boolean;
 begin
  Execute_dfree:= DCalculate (2, D, Index, Add, Data);
 end;

 function Execute_dfs (var Add: byte; var Data: string): boolean;
 var i: byte;
 begin
  Data:= DriveType;
  for i:= 1 to 6 - Length (Data) do Data:= Data + ' ';
  Add:= 3;
  Execute_dfs:= True;
 end;

 function Execute_dpercent (var Add: byte; var Data: string): boolean;
 var Fr: word;
     s1, s2: string;
 begin
  Fr:= Trunc ((SysDiskFreeLong (byte (DriveLetter) - $40) / SysDiskSizeLong (byte (DriveLetter) - $40)) * 10000);
  Str (Fr - (Fr div 100) * 100, s1);
  Str (Fr div 100, s2);
  if Length (s1) = 1 then s1:= '0' + s1;
  if Length (s2) = 1 then s2:= ' ' + s2;
  Data:= s2 + ',' + s1 + '%';
  Add:= 8;
  Execute_dpercent:= True;
 end;

 procedure AnalyzeString (Data: string; A: boolean);
 var i, j   : byte;
     s	    : string;
     Add    : byte;
     ComType: byte;
     State  : boolean;
     WriteS : boolean;
 begin
  s:= '';
  WriteS:= A;
  j:= 0;
  for i:= 1 to Length (Data) do begin
   Inc (j);
   if Data [j] = '@' then if Command (UpStr (Data), j, ComType) then begin
    case ComType of
     cEnd	  : WriteS:= False;
     cIf	  : begin
		     State:= Execute_if (Data, j, A);
		     WriteS:= not State;
		    end;
     cQueueBegin  : begin
		     State:= Execute_queuebegin (A);
		     WriteS:= not State;
		     break;
		    end;
     cDrvInfoBegin: begin
		     State:= Execute_drvinfobegin (Data, j, A);
		     WriteS:= not State;
		    end;
     cQueue	  : begin
		     if not A then Exit;
		     State:= Execute_queue (j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cVersion	  : begin
		     if not A then Exit;
		     State:= Execute_version (j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cCurTime	  : begin
		     if not A then Exit;
		     State:= Execute_time (j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cCurDate	  : begin
		     if not A then Exit;
		     State:= Execute_date (j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDayOfWeek   : begin
		     if not A then Exit;
		     State:= Execute_dayofweek (j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cMainAka	  : begin
		     if not A then Exit;
		     State:= Execute_mainaka (j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cNode	  : begin
		     if not A then Exit;
		     State:= Execute_node (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cMSize	  : begin
		     if not A then Exit;
		     State:= Execute_msize (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cTSize	  : begin
		     if not A then Exit;
		     State:= Execute_tsize (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cPercent	  : begin
		     if not A then Exit;
		     State:= Execute_percent (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cFlags	  : begin
		     if not A then Exit;
		     State:= Execute_flags (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cFDays	  : begin
		     if not A then Exit;
		     State:= Execute_fdays (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cFromDay	  : begin
		     if not A then Exit;
		     State:= Execute_fromday (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cCDays	  : begin
		     if not A then Exit;
		     State:= Execute_cdays (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cCFiles	  : begin
		     if not A then Exit;
		     State:= Execute_cfiles (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cTFiles	  : begin
		     if not A then Exit;
		     State:= Execute_tfiles (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cCDayTime	  : begin
		     if not A then Exit;
		     State:= Execute_cdaytime (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cPFlags	  : begin
		     if not A then Exit;
		     State:= Execute_pflags (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDLetter	  : begin
		     if not A then Exit;
		     State:= Execute_dletter (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDSize	  : begin
		     if not A then Exit;
		     State:= Execute_dsize (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDFree	  : begin
		     if not A then Exit;
		     State:= Execute_dfree (Data, j, Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDLabel	  : begin
		     if not A then Exit;
		     State:= Execute_dlabel (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDFS	  : begin
		     if not A then Exit;
		     State:= Execute_dfs (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;
     cDPercent	  : begin
		     if not A then Exit;
		     State:= Execute_dpercent (Add, e);
		     j:= j + Add;
		     s:= s + e;
		    end;

    end;
    if not State then s:= s + Data [j];
   end else s:= s + Data [j] else s:= s + Data [j];
   if j = Length (Data) then break;
  end;
  if WriteS then AddString (s); {Writeln (s)}
 end;

 procedure ScanTemplate;
 begin
  CalculateTotalMailSize;
  ToLog (0, TemplateGen_);
  while Tpl^.Next <> nil do begin
   ChengetRec:= False;
   AnalyzeString (Tpl^.Str, True);
   if not ChengetRec then Tpl:= Tpl^.Next;
  end;
  AnalyzeString (Tpl^.Str, True);
 end;

 procedure CreateReport;
 var
  TplFile: {$ifndef Win32}text{$else}textfile{$endif};
 begin
  Rep:= nil;
  Tpl:= nil;
  Assign (TplFile, CFG^.Template);
  Reset (TplFile);
  if IOResult <> 0 then begin
   ToLog (0, #16' ' + Cant + TemplateFile + CFG^.Template);
   ErrHalt (esTmplOpen, Cant + TemplateFile + CFG^.Template);
  end;
  while not EOF (TplFile) do begin
   Readln (TplFile, Data);
{$ifndef Win32}
   if not CheckMemory (MemAvail, SizeOf (tStr)) then begin
    ToLog (0, NoMem);
    ToLog (0, InModule_ + Module2);
    Close (TplFile);
    DoneAllModules;
    ErrHalt (esNoMemory, '');
   end;
{$endif}
   if Tpl = nil then begin
    Tpl:= New (pStr_);
    Tpl^.Prev:= nil;
   end else begin
    Tpl^.Next:= New (pStr_);
    Tpl^.Next^.Prev:= Tpl;
    Tpl:= Tpl^.Next;
   end;
   Tpl^.Next:= nil;
   Tpl^.Str:= Data;
  end;
  Close (TplFile);
  while Tpl^.Prev <> nil do Tpl:= Tpl^.Prev;
  ScanTemplate;
 end;

 procedure CloseReport;
 begin
  if Tpl <> nil then begin
   while Tpl^.Prev <> nil do Tpl:= Tpl^.Prev;
   while Tpl^.Next <> nil do begin
    Tpl:= Tpl^.Next;
    Dispose (Tpl^.Prev);
   end;
   Dispose (Tpl);
  end;
  if Rep <> nil then begin
   while Rep^.Prev <> nil do Rep:= Rep^.Prev;
   while Rep^.Next <> nil do begin
    Rep:= Rep^.Next;
    Dispose (Rep^.Prev);
   end;
   Dispose (Rep);
  end;
 end;

{$ifdef Win32}
var
 LibHndl: hInst;

begin
 GDFSEA:= False;
// LibHndl:= LoadLibrary ('kernel32.dll');
 LibHndl:= GetModuleHandle ('kernel32.dll');
 @GetDiskFreeSpaceExA:= GetProcAddress(LibHndl, 'GetDiskFreeSpaceExA');
 if @GetDiskFreeSpaceExA <> nil then GDFSEA:= True;
{$endif}
end.

